@@ -1,5 +1,6 @@
 """ Module defining the Django auth backend class for the Keystone API. """
 
+import hashlib
 import logging
 
 from django.utils.translation import ugettext as _
@@ -10,7 +11,7 @@ from keystoneclient.v2_0.tokens import Token, TokenManager
 
 from .exceptions import KeystoneAuthException
 from .user import create_user_from_token
-from .utils import check_token_expiration
+from .utils import check_token_expiration, is_ans1_token
 
 
 LOG = logging.getLogger(__name__)
@@ -68,7 +69,8 @@ class KeystoneBackend(object):
         except keystone_exceptions.Unauthorized:
             msg = _('Invalid user name or password.')
             raise KeystoneAuthException(msg)
-        except keystone_exceptions.ClientException:
+        except (keystone_exceptions.ClientException,
+                keystone_exceptions.AuthorizationFailure):
             msg = _("An error occurred authenticating. "
                     "Please try again later.")
             raise KeystoneAuthException(msg)
@@ -80,7 +82,8 @@ class KeystoneBackend(object):
         # For now we list all the user's tenants and iterate through.
         try:
             tenants = client.tenants.list()
-        except keystone_exceptions.ClientException:
+        except (keystone_exceptions.ClientException,
+                keystone_exceptions.AuthorizationFailure):
             msg = _('Unable to retrieve authorized projects.')
             raise KeystoneAuthException(msg)
 
@@ -96,7 +99,8 @@ class KeystoneBackend(object):
                                                    token=unscoped_token.id,
                                                    tenant_id=tenant.id)
                 break
-            except keystone_exceptions.ClientException:
+            except (keystone_exceptions.ClientException,
+                    keystone_exceptions.AuthorizationFailure):
                 token = None
 
         if token is None:
@@ -110,6 +114,9 @@ class KeystoneBackend(object):
         user = create_user_from_token(request, token, client.management_url)
 
         if request is not None:
+            if is_ans1_token(unscoped_token.id):
+                hashed_token = hashlib.md5(unscoped_token.id).hexdigest()
+                unscoped_token._info['token']['id'] = hashed_token
             request.session['unscoped_token'] = unscoped_token.id
             request.user = user
 
@@ -118,3 +125,42 @@ class KeystoneBackend(object):
 
         LOG.debug('Authentication completed for user "%s".' % username)
         return user
+
+    def get_group_permissions(self, user, obj=None):
+        """ Returns an empty set since Keystone doesn't support "groups". """
+        return set()
+
+    def get_all_permissions(self, user, obj=None):
+        """
+        Returns a set of permission strings that this user has through his/her
+        Keystone "roles".
+
+        The permissions are returned as ``"openstack.{{ role.name }}"``.
+        """
+        if user.is_anonymous() or obj is not None:
+            return set()
+        # TODO: Integrate policy-driven RBAC when supported by Keystone.
+        role_perms = set(["openstack.roles.%s" % role['name'].lower()
+                          for role in user.roles])
+        service_perms = set(["openstack.services.%s" % service['type'].lower()
+                          for service in user.service_catalog])
+        return role_perms | service_perms
+
+    def has_perm(self, user, perm, obj=None):
+        """ Returns True if the given user has the specified permission. """
+        if not user.is_active:
+            return False
+        return perm in self.get_all_permissions(user, obj)
+
+    def has_module_perms(self, user, app_label):
+        """
+        Returns True if user has any permissions in the given app_label.
+
+        Currently this matches for the app_label ``"openstack"``.
+        """
+        if not user.is_active:
+            return False
+        for perm in self.get_all_permissions(user):
+            if perm[:perm.index('.')] == app_label:
+                return True
+        return False
